# -*- coding: utf-8 -*-
from django.db.models import CharField, TextField

import django_filters
from genomix.filters import DisplayChoiceFilter

from . import choices, models


class FileFilter(django_filters.rest_framework.FilterSet):

    type = DisplayChoiceFilter(choices=choices.FILE_TYPES)

    class Meta:
        model = models.File
        fields = [
            'name',
            'description',
            'url',
            'path',
            'active',
            'type',
            'active',
            'relative_path',
        ]
        filter_overrides = {
            CharField: {
                'filter_class': django_filters.CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'icontains',
                },
            },
            TextField: {
                'filter_class': django_filters.CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'icontains',
                },
            }
        }



# -*- coding: utf-8 -*-
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import viewsets
from rest_framework.filters import SearchFilter

from . import filters, models, serializers


class FileViewSet(viewsets.ModelViewSet):
    """ViewSet for viewing and editing Files."""

    queryset = models.File.objects.all()
    serializer_class = serializers.FileSerializer
    filter_backends = (DjangoFilterBackend, SearchFilter)
    filter_class = filters.FileFilter
    search_fields = (
        'name',
        'description',
        'url',
        'path',
    )



# -*- coding: utf-8 -*-
from django.utils.translation import ugettext_lazy as _
from model_utils import Choices


FILE_TYPES = Choices(
    (0, 'tsv', _('tsv')),
    (1, 'csv', _('csv')),
    (2, 'txt', _('txt')),
    (3, 'vcf', _('vcf')),
    (4, 'bed', _('bed')),
    (5, 'bigwig', _('bigwig')),
    (6, 'wigfix', _('wigfix')),
    (7, 'chromatograms', _('chromatograms')),
    (8, 'other', _('other'))
)



# -*- coding: utf-8 -*-
from genomix.fields import DisplayChoiceField
from rest_framework import serializers

from . import choices, models


class FileSerializer(serializers.ModelSerializer):
    """Serializer for Files."""

    type = DisplayChoiceField(choices=choices.FILE_TYPES)

    class Meta:
        model = models.File
        fields = (
            'id', 'name', 'description', 'url', 'path',
            'type', 'relative_path',
            'active', 'created', 'modified',
        )



# -*- coding: utf-8
from django.contrib import admin

from . import models


class FileAdmin(admin.ModelAdmin):
    model = models.File
    list_display = (
        'name', 'url', 'path', 'relative_path', 'type',
        'active', 'created', 'modified'
    )
    search_fields = ('name', 'description')
    list_filter = ('active', 'relative_path', 'type')
    save_as = True


admin.site.register(models.File, FileAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.8 on 2018-01-09 13:36
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='File',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('name', models.CharField(max_length=255)),
                ('description', models.TextField(blank=True)),
                ('url', models.CharField(max_length=255)),
                ('path', models.CharField(max_length=255)),
                ('type', models.SmallIntegerField(choices=[(0, 'tsv'), (1, 'csv'), (2, 'txt'), (3, 'vcf'), (4, 'bed'), (5, 'bigwig'), (6, 'wigfix'), (7, 'chromatograms'), (8, 'other')])),
                ('active', models.BooleanField(default=True)),
                ('relative_path', models.BooleanField(default=True)),
            ],
            options={
                'verbose_name_plural': 'Files',
                'verbose_name': 'File',
            },
        ),
        migrations.AlterUniqueTogether(
            name='file',
            unique_together=set([('name', 'path'), ('name', 'url')]),
        ),
    ]



# -*- coding: utf-8 -*-
from django.contrib.contenttypes.fields import GenericRelation
from django.db import models
from django.utils.translation import ugettext_lazy as _

from model_utils.models import TimeStampedModel

from . import choices


class File(TimeStampedModel):
    """Model File that stores data relevant.

    Notes:
        - Trying to follow GA4GH data models
        (Specifically https://github.com/ga4gh/task-execution-schemas)
        - Following inputs model
    """
    name = models.CharField(max_length=255)
    description = models.TextField(blank=True)
    url = models.CharField(max_length=255)
    path = models.CharField(max_length=255)
    type = models.SmallIntegerField(
        choices=choices.FILE_TYPES,
    )

    active = models.BooleanField(default=True)
    relative_path = models.BooleanField(default=True)
    comments = GenericRelation('user_activities.Comment')

    class Meta:
        verbose_name = _('File')
        verbose_name_plural = _('Files')
        unique_together = (
            ('name', 'path'),
            ('name', 'url'),
        )

    def __str__(self):
        return self.name
